/*
 * crc14.h
 *
 * CRC-14 0x372b Koopman little-endian checksum (0x3a9d polynomial)
 *
 * Copyright (C) 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifndef __CRC14_H
#define __CRC14_H

#include <asm_types.h>

/*
 * A little-endian version of Koopman 0x372b checksum (0x2e57 polynomial,
 * 0x3a9d after reversing). It's quite good checksum for small messages.
 * In messages up to 57 data bits all errors up to 5 bits are detected.
 *
 * This checksum uses only 7 bits per character, it's designed to
 * checksum 8 characters, each with 7 bits, 56 data bits total.
 */

extern const u16 crc14_table[128];

static inline u16 crc14_byte(u16 crc, const u8 data)
{
	return (crc >> 7) ^ crc14_table[(crc ^ (data & 0x7f)) & 0x7f];
}

u16 crc14(u16 crc, const u8 *buf, unsigned char len);

#endif
